@@ -1,15 +1,22 @@
 """Module defining core VASP input set generators."""
 
+from __future__ import annotations
+
+import logging
 from copy import deepcopy
 from dataclasses import dataclass, field
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any
 
 import numpy as np
 from pymatgen.core import Structure
+from pymatgen.core.periodic_table import Element
 from pymatgen.io.vasp import Outcar, Vasprun
 
 from atomate2.common.schemas.math import Vector3D
-from atomate2.vasp.sets.base import VaspInputSetGenerator
+from atomate2.vasp.sets.base import VaspInputGenerator
+
+logger = logging.getLogger(__name__)
+
 
 __all__ = [
     "RelaxSetGenerator",
@@ -21,11 +28,12 @@ __all__ = [
     "HSEBSSetGenerator",
     "HSETightRelaxSetGenerator",
     "ElectronPhononSetGenerator",
+    "MDSetGenerator",
 ]
 
 
 @dataclass
-class RelaxSetGenerator(VaspInputSetGenerator):
+class RelaxSetGenerator(VaspInputGenerator):
     """Class to generate VASP relaxation input sets."""
 
     def get_incar_updates(
@@ -61,7 +69,7 @@ class RelaxSetGenerator(VaspInputSetGenerator):
 
 
 @dataclass
-class TightRelaxSetGenerator(VaspInputSetGenerator):
+class TightRelaxSetGenerator(VaspInputGenerator):
     """Class to generate tight VASP relaxation input sets."""
 
     def get_incar_updates(
@@ -107,7 +115,7 @@ class TightRelaxSetGenerator(VaspInputSetGenerator):
 
 
 @dataclass
-class StaticSetGenerator(VaspInputSetGenerator):
+class StaticSetGenerator(VaspInputGenerator):
     """
     Class to generate VASP static input sets.
 
@@ -120,7 +128,7 @@ class StaticSetGenerator(VaspInputSetGenerator):
         Whether to set LCALCPOL (used for calculating the electronic contribution to
         the polarization)
     **kwargs
-        Other keyword arguments that will be passed to :obj:`VaspInputSetGenerator`.
+        Other keyword arguments that will be passed to :obj:`VaspInputGenerator`.
     """
 
     lepsilon: bool = False
@@ -174,7 +182,7 @@ class StaticSetGenerator(VaspInputSetGenerator):
 
 
 @dataclass
-class NonSCFSetGenerator(VaspInputSetGenerator):
+class NonSCFSetGenerator(VaspInputGenerator):
     """
     Class to generate VASP non-self-consistent field input sets.
 
@@ -194,7 +202,7 @@ class NonSCFSetGenerator(VaspInputSetGenerator):
         Multiplicative factor for NBANDS when starting from a previous calculation.
         Choose a higher number if you are doing an LOPTICS calculation.
     **kwargs
-        Other keyword arguments that will be passed to :obj:`VaspInputSetGenerator`.
+        Other keyword arguments that will be passed to :obj:`VaspInputGenerator`.
     """
 
     mode: str = "line"
@@ -282,7 +290,7 @@ class NonSCFSetGenerator(VaspInputSetGenerator):
         dict
             A dictionary of updates to apply.
         """
-        updates: Dict[str, Any] = {
+        updates: dict[str, Any] = {
             "LCHARG": False,
             "LORBIT": 11,
             "LWAVE": False,
@@ -322,8 +330,15 @@ class NonSCFSetGenerator(VaspInputSetGenerator):
 
 
 @dataclass
-class HSERelaxSetGenerator(VaspInputSetGenerator):
-    """Class to generate VASP HSE06 relaxation input sets."""
+class HSERelaxSetGenerator(VaspInputGenerator):
+    """Class to generate VASP HSE06 relaxation input sets.
+
+    .. note::
+        By default the hybrid input sets use ALGO = Normal which is only efficient for
+        VASP 6.0 and higher. See https://www.vasp.at/wiki/index.php/LFOCKACE for more
+        details.
+
+    """
 
     def get_incar_updates(
         self,
@@ -356,7 +371,7 @@ class HSERelaxSetGenerator(VaspInputSetGenerator):
         """
         return {
             "NSW": 99,
-            "ALGO": "All",
+            "ALGO": "Normal",
             "GGA": "PE",
             "HFSCREEN": 0.2,
             "LHFCALC": True,
@@ -369,8 +384,14 @@ class HSERelaxSetGenerator(VaspInputSetGenerator):
 
 
 @dataclass
-class HSETightRelaxSetGenerator(VaspInputSetGenerator):
-    """Class to generate tight VASP HSE relaxation input sets."""
+class HSETightRelaxSetGenerator(VaspInputGenerator):
+    """Class to generate tight VASP HSE relaxation input sets.
+
+    .. note::
+        By default the hybrid input sets use ALGO = Normal which is only efficient for
+        VASP 6.0 and higher. See https://www.vasp.at/wiki/index.php/LFOCKACE for more
+        details.
+    """
 
     def get_incar_updates(
         self,
@@ -409,7 +430,7 @@ class HSETightRelaxSetGenerator(VaspInputSetGenerator):
             "LAECHG": False,
             "EDIFFG": -0.001,
             "LREAL": False,
-            "ALGO": "All",
+            "ALGO": "Normal",
             "NSW": 99,
             "LCHARG": False,
             "GGA": "PE",
@@ -422,8 +443,15 @@ class HSETightRelaxSetGenerator(VaspInputSetGenerator):
 
 
 @dataclass
-class HSEStaticSetGenerator(VaspInputSetGenerator):
-    """Class to generate VASP HSE06 static input sets."""
+class HSEStaticSetGenerator(VaspInputGenerator):
+    """Class to generate VASP HSE06 static input sets.
+
+    .. note::
+        By default the hybrid input sets use ALGO = Normal which is only efficient for
+        VASP 6.0 and higher. See https://www.vasp.at/wiki/index.php/LFOCKACE for more
+        details.
+
+    """
 
     def get_incar_updates(
         self,
@@ -456,7 +484,7 @@ class HSEStaticSetGenerator(VaspInputSetGenerator):
         """
         return {
             "NSW": 0,
-            "ALGO": "All",
+            "ALGO": "Normal",
             "GGA": "PE",
             "HFSCREEN": 0.2,
             "LHFCALC": True,
@@ -471,7 +499,7 @@ class HSEStaticSetGenerator(VaspInputSetGenerator):
 
 
 @dataclass
-class HSEBSSetGenerator(VaspInputSetGenerator):
+class HSEBSSetGenerator(VaspInputGenerator):
     """
     Class to generate VASP HSE06 band structure input sets.
 
@@ -493,6 +521,11 @@ class HSEBSSetGenerator(VaspInputSetGenerator):
     The "uniform_dense" mode employs are regular weighted k-point mesh, in addition
     to a zero-weighted uniform mesh with higher density.
 
+    .. note::
+        By default the hybrid input sets use ALGO = Normal which is only efficient for
+        VASP 6.0 and higher. See https://www.vasp.at/wiki/index.php/LFOCKACE for more
+        details.
+
     Parameters
     ----------
     mode
@@ -514,17 +547,17 @@ class HSEBSSetGenerator(VaspInputSetGenerator):
     added_kpoints
         A list of kpoints in fractional coordinates to add as zero-weighted points.
     **kwargs
-        Other keyword arguments that will be passed to :obj:`VaspInputSetGenerator`.
+        Other keyword arguments that will be passed to :obj:`VaspInputGenerator`.
     """
 
     mode: str = "gap"
     dedos: float = 0.02
-    reciprocal_density: float = 50
+    reciprocal_density: float = 64
     line_density: float = 20
     zero_weighted_reciprocal_density: float = 100
     optics: bool = False
     nbands_factor: float = 1.2
-    added_kpoints: List[Vector3D] = field(default_factory=list)
+    added_kpoints: list[Vector3D] = field(default_factory=list)
     auto_ispin: bool = True
 
     def __post_init__(self):
@@ -567,7 +600,7 @@ class HSEBSSetGenerator(VaspInputSetGenerator):
         dict
             A dictionary of updates to apply to the KPOINTS config.
         """
-        kpoints: Dict[str, Any] = {"reciprocal_density": self.reciprocal_density}
+        kpoints: dict[str, Any] = {"reciprocal_density": self.reciprocal_density}
 
         if self.mode == "line":
             # add line_density on top of reciprocal density
@@ -620,7 +653,7 @@ class HSEBSSetGenerator(VaspInputSetGenerator):
         """
         updates = {
             "NSW": 0,
-            "ALGO": "All",
+            "ALGO": "Normal",
             "GGA": "PE",
             "HFSCREEN": 0.2,
             "PRECFOCK": "Fast",
@@ -660,7 +693,7 @@ class HSEBSSetGenerator(VaspInputSetGenerator):
 
 
 @dataclass
-class ElectronPhononSetGenerator(VaspInputSetGenerator):
+class ElectronPhononSetGenerator(VaspInputGenerator):
     """
     Class to generate VASP electron phonon input sets.
 
@@ -676,7 +709,7 @@ class ElectronPhononSetGenerator(VaspInputSetGenerator):
         Density of k-mesh by reciprocal volume.
     """
 
-    temperatures: Tuple[float, ...] = (
+    temperatures: tuple[float, ...] = (
         0,
         100,
         200,
@@ -775,7 +808,122 @@ class ElectronPhononSetGenerator(VaspInputSetGenerator):
         return {"reciprocal_density": self.reciprocal_density}
 
 
-def _get_nedos(vasprun: Optional[Vasprun], dedos: float):
+@dataclass
+class MDSetGenerator(VaspInputGenerator):
+    """
+    Class to generate VASP molecular dynamics input sets.
+
+    Parameters
+    ----------
+    ensemble
+        Molecular dynamics ensemble to run. Options include `nvt`, `nve`, and `npt`.
+    start_temp
+        Starting temperature. The VASP `TEBEG` parameter.
+    end_temp
+        Final temperature. The VASP `TEEND` parameter.
+    nsteps
+        Number of time steps for simulations. The VASP `NSW` parameter.
+    time_step
+        The time step (in femtosecond) for the simulation. The VASP `POTIM` parameter.
+    **kwargs
+        Other keyword arguments that will be passed to :obj:`VaspInputGenerator`.
+    """
+
+    ensemble: str = "nvt"
+    start_temp: float = 300
+    end_temp: float = 300
+    nsteps: int = 1000
+    time_step: int = 2
+    auto_ispin: bool = True
+
+    def get_incar_updates(
+        self,
+        structure: Structure,
+        prev_incar: dict = None,
+        bandgap: float = 0,
+        vasprun: Vasprun = None,
+        outcar: Outcar = None,
+    ) -> dict:
+        """
+        Get updates to the INCAR for a molecular dynamics job.
+
+        Parameters
+        ----------
+        structure
+            A structure.
+        prev_incar
+            An incar from a previous calculation.
+        bandgap
+            The band gap.
+        vasprun
+            A vasprun from a previous calculation.
+        outcar
+            An outcar from a previous calculation.
+
+        Returns
+        -------
+        dict
+            A dictionary of updates to apply.
+        """
+        updates = self._get_ensemble_defaults(structure, self.ensemble)
+
+        # Based on pymatgen.io.vasp.sets.MPMDSet.
+        updates.update(
+            {
+                "ENCUT": 520,
+                "TEBEG": self.start_temp,
+                "TEEND": self.end_temp,
+                "NSW": self.nsteps,
+                "POTIM": self.time_step,
+                "LCHARG": False,
+                "NELMIN": 4,
+                "MAXMIX": 20,
+                "NELM": 500,
+                "ISYM": 0,
+                "IBRION": 0,
+                "KBLOCK": 100,
+                "PREC": "Normal",
+            }
+        )
+
+        if Element("H") in structure.species:
+            if updates["POTIM"] > 0.5:
+                logger.warning(
+                    f"Molecular dynamics time step is {updates['POTIM']}, which is "
+                    "typically too large for a structure containing H. Consider set it "
+                    "to a value of 0.5 or smaller."
+                )
+
+        return updates
+
+    @staticmethod
+    def _get_ensemble_defaults(structure: Structure, ensemble: str) -> dict[str, Any]:
+        """
+        Get default params for the ensemble.
+        """
+        defaults = {
+            "nve": {"MDALGO": 1, "ISIF": 2, "ANDERSEN_PROB": 0.0},
+            "nvt": {"MDALGO": 2, "ISIF": 2, "SMASS": 0},
+            "npt": {
+                "MDALGO": 3,
+                "ISIF": 3,
+                "LANGEVIN_GAMMA": [10] * structure.ntypesp,
+                "LANGEVIN_GAMMA_L": 1,
+                "PMASS": 10,
+                "PSTRESS": 0,
+            },
+        }
+
+        try:
+            return defaults[ensemble.lower()]  # type: ignore
+        except KeyError:
+            supported = tuple(defaults.keys())
+            raise ValueError(
+                f"Expect `ensemble` to be one of {supported}; got {ensemble}."
+            )
+
+
+def _get_nedos(vasprun: Vasprun | None, dedos: float):
     """Automatic setting of nedos using the energy range and the energy step."""
     if vasprun is None:
         return 2000
